@@ -1,24 +1,45 @@
 """The IPython kernel implementation"""
 
+import asyncio
+from contextlib import contextmanager
+from functools import partial
 import getpass
+import signal
 import sys
-import traceback
 
 from IPython.core import release
 from ipython_genutils.py3compat import builtin_mod, PY3, unicode_type, safe_unicode
 from IPython.utils.tokenutil import token_at_cursor, line_at_cursor
-from traitlets import Instance, Type, Any, List
+from tornado import gen
+from traitlets import Instance, Type, Any, List, Bool
 
 from .comm import CommManager
 from .kernelbase import Kernel as KernelBase
 from .zmqshell import ZMQInteractiveShell
 
+try:
+    from IPython.core.interactiveshell import _asyncio_runner
+except ImportError:
+    _asyncio_runner = None
+
+try:
+    from IPython.core.completer import rectify_completions as _rectify_completions, provisionalcompleter as _provisionalcompleter
+    _use_experimental_60_completion = True
+except ImportError:
+    _use_experimental_60_completion = False
+
+_EXPERIMENTAL_KEY_NAME = '_jupyter_types_experimental'
+
 
 class IPythonKernel(KernelBase):
     shell = Instance('IPython.core.interactiveshell.InteractiveShellABC',
                      allow_none=True)
     shell_class = Type(ZMQInteractiveShell)
 
+    use_experimental_completions = Bool(True,
+        help="Set this flag to False to deactivate the use of experimental IPython completion APIs.",
+    ).tag(config=True)
+
     user_module = Any()
     def _user_module_changed(self, name, old, new):
         if self.shell is not None:
@@ -60,32 +81,32 @@ class IPythonKernel(KernelBase):
 
     help_links = List([
         {
-            'text': "Python",
-            'url': "http://docs.python.org/%i.%i" % sys.version_info[:2],
+            'text': "Python Reference",
+            'url': "https://docs.python.org/%i.%i" % sys.version_info[:2],
         },
         {
-            'text': "IPython",
-            'url': "http://ipython.org/documentation.html",
+            'text': "IPython Reference",
+            'url': "https://ipython.org/documentation.html",
         },
         {
-            'text': "NumPy",
-            'url': "http://docs.scipy.org/doc/numpy/reference/",
+            'text': "NumPy Reference",
+            'url': "https://docs.scipy.org/doc/numpy/reference/",
         },
         {
-            'text': "SciPy",
-            'url': "http://docs.scipy.org/doc/scipy/reference/",
+            'text': "SciPy Reference",
+            'url': "https://docs.scipy.org/doc/scipy/reference/",
         },
         {
-            'text': "Matplotlib",
-            'url': "http://matplotlib.org/contents.html",
+            'text': "Matplotlib Reference",
+            'url': "https://matplotlib.org/contents.html",
         },
         {
-            'text': "SymPy",
+            'text': "SymPy Reference",
             'url': "http://docs.sympy.org/latest/index.html",
         },
         {
-            'text': "pandas",
-            'url': "http://pandas.pydata.org/pandas-docs/stable/",
+            'text': "pandas Reference",
+            'url': "https://pandas.pydata.org/pandas-docs/stable/",
         },
     ]).tag(config=True)
 
@@ -181,10 +202,58 @@ class IPythonKernel(KernelBase):
 
     @execution_count.setter
     def execution_count(self, value):
-        # Ignore the incrememnting done by KernelBase, in favour of our shell's
+        # Ignore the incrementing done by KernelBase, in favour of our shell's
         # execution counter.
         pass
 
+    @contextmanager
+    def _cancel_on_sigint(self, future):
+        """ContextManager for capturing SIGINT and cancelling a future
+
+        SIGINT raises in the event loop when running async code,
+        but we want it to halt a coroutine.
+
+        Ideally, it would raise KeyboardInterrupt,
+        but this turns it into a CancelledError.
+        At least it gets a decent traceback to the user.
+        """
+        sigint_future = asyncio.Future()
+
+        # whichever future finishes first,
+        # cancel the other one
+        def cancel_unless_done(f, _ignored):
+            if f.cancelled() or f.done():
+                return
+            f.cancel()
+
+        # when sigint finishes,
+        # abort the coroutine with CancelledError
+        sigint_future.add_done_callback(
+            partial(cancel_unless_done, future)
+        )
+        # when the main future finishes,
+        # stop watching for SIGINT events
+        future.add_done_callback(
+            partial(cancel_unless_done, sigint_future)
+        )
+
+        def handle_sigint(*args):
+            def set_sigint_result():
+                if sigint_future.cancelled() or sigint_future.done():
+                    return
+                sigint_future.set_result(1)
+            # use add_callback for thread safety
+            self.io_loop.add_callback(set_sigint_result)
+
+        # set the custom sigint hander during this context
+        save_sigint = signal.signal(signal.SIGINT, handle_sigint)
+        try:
+            yield
+        finally:
+            # restore the previous sigint handler
+            signal.signal(signal.SIGINT, save_sigint)
+
+    @gen.coroutine
     def do_execute(self, code, silent, store_history=True,
                    user_expressions=None, allow_stdin=False):
         shell = self.shell # we'll need this a lot here
@@ -192,8 +261,37 @@ class IPythonKernel(KernelBase):
         self._forward_input(allow_stdin)
 
         reply_content = {}
+        if hasattr(shell, 'run_cell_async') and hasattr(shell, 'should_run_async'):
+            run_cell = shell.run_cell_async
+            should_run_async = shell.should_run_async
+        else:
+            should_run_async = lambda cell: False
+            # older IPython,
+            # use blocking run_cell and wrap it in coroutine
+            @gen.coroutine
+            def run_cell(*args, **kwargs):
+                return shell.run_cell(*args, **kwargs)
         try:
-            res = shell.run_cell(code, store_history=store_history, silent=silent)
+
+            # default case: runner is asyncio and asyncio is already running
+            # TODO: this should check every case for "are we inside the runner",
+            # not just asyncio
+            if (
+                _asyncio_runner
+                and should_run_async(code)
+                and shell.loop_runner is _asyncio_runner
+                and asyncio.get_event_loop().is_running()
+            ):
+                coro = run_cell(code, store_history=store_history, silent=silent)
+                coro_future = asyncio.ensure_future(coro)
+
+                with self._cancel_on_sigint(coro_future):
+                    res = yield coro_future
+            else:
+                # runner isn't already running,
+                # make synchronous call,
+                # letting shell dispatch to loop runners
+                res = shell.run_cell(code, store_history=store_history, silent=silent)
         finally:
             self._restore_input()
 
@@ -213,7 +311,7 @@ class IPythonKernel(KernelBase):
                 u'evalue': safe_unicode(err),
             })
 
-            # FIXME: deprecate piece for ipyparallel:
+            # FIXME: deprecated piece for ipyparallel (remove in 5.0):
             e_info = dict(engine_uuid=self.ident, engine_id=self.int_id,
                           method='execute')
             reply_content['engine_info'] = e_info
@@ -246,6 +344,9 @@ class IPythonKernel(KernelBase):
         return reply_content
 
     def do_complete(self, code, cursor_pos):
+        if _use_experimental_60_completion and self.use_experimental_completions:
+            return self._experimental_do_complete(code, cursor_pos)
+
         # FIXME: IPython completers currently assume single line,
         # but completion messages give multi-line context
         # For now, extract line from cell, based on cursor_pos:
@@ -261,20 +362,60 @@ class IPythonKernel(KernelBase):
                 'metadata' : {},
                 'status' : 'ok'}
 
+    def _experimental_do_complete(self, code, cursor_pos):
+        """
+        Experimental completions from IPython, using Jedi. 
+        """
+        if cursor_pos is None:
+            cursor_pos = len(code)
+        with _provisionalcompleter():
+            raw_completions = self.shell.Completer.completions(code, cursor_pos)
+            completions = list(_rectify_completions(code, raw_completions))
+            
+            comps = []
+            for comp in completions:
+                comps.append(dict(
+                            start=comp.start,
+                            end=comp.end,
+                            text=comp.text,
+                            type=comp.type,
+                ))
+
+        if completions:
+            s = completions[0].start
+            e = completions[0].end
+            matches = [c.text for c in completions]
+        else:
+            s = cursor_pos
+            e = cursor_pos
+            matches = []
+
+        return {'matches': matches,
+                'cursor_end': e,
+                'cursor_start': s,
+                'metadata': {_EXPERIMENTAL_KEY_NAME: comps},
+                'status': 'ok'}
+
+
+
     def do_inspect(self, code, cursor_pos, detail_level=0):
         name = token_at_cursor(code, cursor_pos)
-        info = self.shell.object_inspect(name)
 
         reply_content = {'status' : 'ok'}
-        reply_content['data'] = data = {}
+        reply_content['data'] = {}
         reply_content['metadata'] = {}
-        reply_content['found'] = info['found']
-        if info['found']:
-            info_text = self.shell.object_inspect_text(
-                name,
-                detail_level=detail_level,
+        try:
+            reply_content['data'].update(
+                self.shell.object_inspect_mime(
+                    name,
+                    detail_level=detail_level
+                )
             )
-            data['text/plain'] = info_text
+            if not self.shell.enable_html_pager:
+                reply_content['data'].pop('text/html')
+            reply_content['found'] = True
+        except KeyError:
+            reply_content['found'] = False
 
         return reply_content
 
@@ -304,7 +445,7 @@ class IPythonKernel(KernelBase):
         return dict(status='ok', restart=restart)
 
     def do_is_complete(self, code):
-        status, indent_spaces = self.shell.input_transformer_manager.check_complete(code)
+        status, indent_spaces = self.shell.input_splitter.check_complete(code)
         r = {'status': status}
         if status == 'incomplete':
             r['indent'] = ' ' * indent_spaces
@@ -351,7 +492,7 @@ class IPythonKernel(KernelBase):
                 u'ename': unicode_type(type(e).__name__),
                 u'evalue': safe_unicode(e),
             }
-            # FIXME: deprecate piece for ipyparallel:
+            # FIXME: deprecated piece for ipyparallel (remove in 5.0):
             e_info = dict(engine_uuid=self.ident, engine_id=self.int_id, method='apply')
             reply_content['engine_info'] = e_info
 
